"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var NormalAction_1 = require("./action/NormalAction");
var BaseReducer_1 = require("./reducer/BaseReducer");
var createReduxStore_1 = require("./utils/createReduxStore");
var dev_1 = require("./utils/dev");
var dev_2 = require("../libs/dev");
var ForgetRegisterError_1 = require("./exceptions/ForgetRegisterError");
var NullReducerError_1 = require("./exceptions/NullReducerError");
var HttpServiceHandle_1 = require("./service/HttpServiceHandle");
var instanceName_1 = require("./utils/instanceName");
var method_1 = require("./utils/method");
var BaseModel = /** @class */ (function () {
    function BaseModel(alias) {
        var _this = this;
        if (alias === void 0) { alias = ''; }
        this.reducerHasEffects = false;
        this.instanceName = instanceName_1.setInstanceName(this.constructor.name, alias);
        this.onInit();
        createReduxStore_1.onStoreCreated(function (store) {
            _this.onReducerCreated(store);
        });
        if (this.autoRegister()) {
            createReduxStore_1.appendReducers(this.register());
            if (this.reducer && this.reducerHasEffects) {
                createReduxStore_1.watchEffectsReducer(this.instanceName, this.constructor.name);
            }
        }
        if (dev_2.isDebug() && dev_1.isProxyEnable()) {
            // Proxy is es6 syntax, and it can't be transformed to es5.
            return new Proxy(this, {
                set: function (model, property, value) {
                    model[property] = value;
                    if (typeof value === 'function' && value.__isAction__ === true) {
                        var instance = value;
                        instance.setActionName(property);
                    }
                    return true;
                },
            });
        }
    }
    // As we know, it's forbidden to make condition when we are using hooks.
    // We can't write code like: xxxModel.xxx.useLoading() || xxxModel.yyy.useLoading()
    // So, just write code like: Model.isLoading(xxxModel.xxx.useLoading(), xxxModel.yyy.useLoading());
    BaseModel.isLoading = function () {
        var fromUseLoading = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            fromUseLoading[_i] = arguments[_i];
        }
        return fromUseLoading.some(function (is) { return is; });
    };
    BaseModel.prototype.register = function () {
        var _a;
        // Only create class once.
        // For effects model, register() will be invoked twice.
        if (!this.reducer) {
            var initData = this.initReducer();
            if (initData !== null) {
                this.reducer = new BaseReducer_1.BaseReducer(initData, this.instanceName);
            }
        }
        if (this.reducer) {
            var sideEffects = this.effects();
            this.reducer.clear();
            (_a = this.reducer).addCase.apply(_a, tslib_1.__spread(sideEffects));
            this.reducerHasEffects = sideEffects.length > 0;
            return this.reducer.createData();
        }
        return {};
    };
    BaseModel.prototype.useData = function (filter) {
        var _this = this;
        if (this.reducer) {
            return this.switchReduxSelector()(function (state) {
                var customData = state[_this.instanceName];
                if (customData === undefined) {
                    throw new ForgetRegisterError_1.ForgetRegisterError(_this.constructor.name);
                }
                return filter ? filter(customData) : customData;
            });
        }
        throw new NullReducerError_1.NullReducerError(this.constructor.name);
    };
    Object.defineProperty(BaseModel.prototype, "data", {
        get: function () {
            if (this.reducer) {
                var customData = this.reducer.getData();
                if (customData === undefined) {
                    throw new ForgetRegisterError_1.ForgetRegisterError(this.constructor.name);
                }
                return customData;
            }
            throw new NullReducerError_1.NullReducerError(this.constructor.name);
        },
        enumerable: true,
        configurable: true
    });
    BaseModel.prototype.onInit = function () {
        // Do anything as in constructor.
    };
    BaseModel.prototype.onReducerCreated = function (_store) {
        // Do anything after reducer is generated.
    };
    BaseModel.prototype.changeReducer = function (fn) {
        if (this.reducer) {
            if (this.change_reducer) {
                this.change_reducer.changeEffect(fn);
            }
            else {
                this.change_reducer = this.action(fn);
            }
            this.change_reducer();
        }
        else {
            throw new NullReducerError_1.NullReducerError(this.instanceName);
        }
    };
    BaseModel.prototype.action = function (changeReducer) {
        var _this = this;
        var instanceName = instanceName_1.getInstanceName();
        if (!dev_2.isDebug() || !dev_1.isProxyEnable()) {
            instanceName += '_' + instanceName_1.increaseActionCounter();
        }
        return new NormalAction_1.NormalAction(
        // @ts-ignore
        function (payload) {
            var normal = {
                type: '',
                payload: payload === undefined ? {} : payload,
                reducerName: _this.instanceName,
                effect: function (state, action) {
                    return changeReducer(state, action.payload);
                },
            };
            return normal;
        }, instanceName);
    };
    BaseModel.prototype.serviceAction = function (uri, method) {
        // @ts-ignore
        return new HttpServiceHandle_1.HttpServiceHandle({
            uri: uri,
            method: method,
            instanceName: this.instanceName,
        });
    };
    BaseModel.prototype.get = function (uri) {
        return this.serviceAction(uri, method_1.METHOD.get);
    };
    BaseModel.prototype.post = function (uri) {
        return this.serviceAction(uri, method_1.METHOD.post);
    };
    BaseModel.prototype.put = function (uri) {
        return this.serviceAction(uri, method_1.METHOD.put);
    };
    BaseModel.prototype.delete = function (uri) {
        return this.serviceAction(uri, method_1.METHOD.delete);
    };
    BaseModel.prototype.effects = function () {
        return [];
    };
    BaseModel.prototype.autoRegister = function () {
        return true;
    };
    return BaseModel;
}());
exports.BaseModel = BaseModel;
